%define spu_version 3.2
%define spu_release 6
%define gcc_version 4.0.2
%define _unpackaged_files_terminate_build 1
Summary: SPU compiler and toolchain
Name: spu-toolchain
Version: %{spu_version}
Release: %{spu_release}
License: GPL
Group: Development/Languages
Source0: toolchain-%{spu_version}-src.tar.bz2
Source1: gdb-6.3.tar.bz2
Patch1: toolchain-binutils-overflow.diff
Patch2: toolchain-libiberty-merge.diff
Patch3: toolchain-mfcio-stdint.diff
Patch8: toolchain-convertible-revert.diff
Patch10: diff-spu-gdb
Patch11: toolchain-build-revert.diff 
Patch12: toolchain-newlib-setjmp-longjmp.diff
Patch20: diff-spu-gdb-fixes
BuildRoot: /var/tmp/spu-root
BuildRequires: zlib-devel, dejagnu, bison, flex, texinfo

# Don't strip as the host binutils may be unable to handle SPU objects.
%define __os_install_post /usr/lib/rpm/brp-compress

# When building the cross-toolchain, reset prefix directory
%ifnarch ppc ppc64
%define _prefix /opt/sce/toolchain-%{version}/spu
%define _mandir %{_prefix}/man
%define _infodir %{_prefix}/info
%endif

%description
The spu-toolchain package contains a development tool chain for the
Cell Broadband Engine SPU architecture, consisting of the GNU Compiler
Collection version 4.0.2, GNU binutils version 2.16.1, and GNU newlib
version 1.12.0.

%package -n spu-gcc
Summary: SPU cross-compiler
Group: Development/Languages
Requires: spu-binutils = %{version}-%{release}
AutoProv: no
AutoReq: no

%description -n spu-gcc
GNU Compiler Collection C compiler for Cell Broadband Engine SPU.

%package -n spu-gcc-c++
Summary: C++ support for SPU cross-compiler
Group: Development/Languages
Requires: spu-gcc = %{version}-%{release}
AutoProv: no
AutoReq: no

%description -n spu-gcc-c++
GNU Compiler Collection C++ compiler for Cell Broadband Engine SPU.

%package -n spu-binutils
Summary: SPU cross-binutils
Group: Development/Tools
AutoProv: no
AutoReq: no

%description -n spu-binutils
GNU binary utilities for Cell Broadband Engine SPU.

%package -n spu-newlib
Summary: SPU runtime library
Group: Development/Tools
AutoProv: no
AutoReq: no

%description -n spu-newlib
GNU newlib runtime library for Cell Broadband Engine SPU.

%package -n spu-gdb
Summary: SPU debugger
Group: Development/Tools
AutoProv: yes
AutoReq: yes

%description -n spu-gdb
GNU debugger for Cell Broadband Engine SPU (standalone).

%prep
%setup -c -q -a 1
%patch1 -p0
%patch2 -p0
%patch3 -p0
%patch8 -p0
%patch10 -p0
%patch11 -p0
%patch12 -p0
%patch20 -p0

pushd toolchain/gcc
./contrib/gcc_update --touch
popd

# GDB directories don't work with symlinks for some reason
mv gdb-6.3/{gdb,readline} toolchain/binutils

# Set up combined tree
rm -fr src
mkdir src
for I in toolchain/newlib/* toolchain/binutils/* toolchain/gcc/*; do
  ln -sf ../$I src
done

%build

%define spu_target_platform spu
%define spu_prefix spu-

rm -fr obj-%{spu_target_platform}
mkdir obj-%{spu_target_platform}
cd obj-%{spu_target_platform}

OPT_FLAGS=`echo $RPM_OPT_FLAGS|sed -e 's/-fno-rtti//g' -e 's/-fno-exceptions//g'`
SPU_OPT_FLAGS="-O2 -g -pipe"
CC="gcc $OPT_FLAGS -DBPA"
CC="$CC" CFLAGS="$SPU_OPT_FLAGS" CXXFLAGS="$SPU_OPT_FLAGS" XCFLAGS="$OPT_FLAGS" TCFLAGS="$SPU_OPT_FLAGS" \
	../src/configure --prefix=%{_prefix} --mandir=%{_mandir} --infodir=%{_infodir} \
	--disable-shared --disable-threads --disable-checking --with-headers \
	--with-system-zlib --with-newlib \
	--enable-languages=c,c++ --disable-multilib --disable-nls \
	--enable-version-specific-runtime-libs \
	--program-prefix=%{spu_prefix} \
	--target=%{spu_target_platform}

make %{?_smp_mflags}

%install
rm -fr $RPM_BUILD_ROOT

cd obj-%{spu_target_platform}
TARGET_PLATFORM=%{spu_target_platform}

# There are some MP bugs in libstdc++ and libjava Makefiles
make -C %{spu_target_platform}/libstdc++-v3

make DESTDIR=$RPM_BUILD_ROOT install

FULLPATH=$RPM_BUILD_ROOT%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}

# fix some things
ln -sf %{spu_prefix}gcc $RPM_BUILD_ROOT%{_prefix}/bin/%{spu_prefix}cc
#rm -f $RPM_BUILD_ROOT%{_infodir}/dir
#gzip -9 $RPM_BUILD_ROOT%{_infodir}/*.info*
rm -f $RPM_BUILD_ROOT%{_prefix}/bin/%{spu_prefix}gcc-%{gcc_version}
rm -f $RPM_BUILD_ROOT%{_prefix}/bin/%{spu_prefix}gccbug
rm -f $RPM_BUILD_ROOT%{_prefix}/lib/libiberty.a
rm -f $RPM_BUILD_ROOT%{_prefix}/lib/libmmalloc.a
rm -f $RPM_BUILD_ROOT%{_prefix}/%{spu_target_platform}/lib/libiberty.a

rm -rf $RPM_BUILD_ROOT%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/install-tools
rm -rf $RPM_BUILD_ROOT%{_prefix}/libexec/gcc/%{spu_target_platform}/%{gcc_version}/install-tools

rm -f $RPM_BUILD_ROOT%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/*.la

for h in `find $FULLPATH/include -name \*.h`; do
  if grep -q 'It has been auto-edited by fixincludes from' $h; then
    rh=`grep -A2 'It has been auto-edited by fixincludes from' $h | tail -1 | sed 's|^.*"\(.*\)".*$|\1|'`
    diff -up $rh $h || :
    rm -f $h
  fi
done

cat > $RPM_BUILD_ROOT%{_prefix}/bin/%{spu_prefix}c89 <<"EOF"
#!/bin/sh
fl="-std=c89"
for opt; do
  case "$opt" in
    -ansi|-std=c89|-std=iso9899:1990) fl="";;
    -std=*) echo "`basename $0` called with non ANSI/ISO C option $opt" >&2
	    exit 1;;
  esac
done
exec %{_prefix}/bin/%{spu_prefix}gcc $fl ${1+"$@"}
EOF
cat > $RPM_BUILD_ROOT%{_prefix}/bin/%{spu_prefix}c99 <<"EOF"
#!/bin/sh
fl="-std=c99"
for opt; do
  case "$opt" in
    -std=c99|-std=iso9899:1999) fl="";;
    -std=*) echo "`basename $0` called with non ISO C99 option $opt" >&2
	    exit 1;;
  esac
done
exec %{_prefix}/bin/%{spu_prefix}gcc $fl ${1+"$@"}
EOF
chmod 755 $RPM_BUILD_ROOT%{_prefix}/bin/%{spu_prefix}c?9

# Remove Windows/Novell only man pages
rm -f $RPM_BUILD_ROOT%{_mandir}/man1/%{spu_prefix}{dlltool,nlmconv,windres}*

# Info and man7 files conflict with host toolchain files, 
# just remove them for now.
rm -rf $RPM_BUILD_ROOT%{_infodir}
rm -rf $RPM_BUILD_ROOT%{_mandir}/man7

cd ..

# Install the embedspu script
cp toolchain/scripts/embedspu.sh $RPM_BUILD_ROOT%{_prefix}/bin
chmod 755 $RPM_BUILD_ROOT%{_prefix}/bin/embedspu.sh
ln -sf embedspu.sh $RPM_BUILD_ROOT%{_prefix}/bin/embedspu

%clean
rm -rf $RPM_BUILD_ROOT

%files -n spu-gcc
%defattr(-,root,root)
%dir %{_prefix}
%dir %{_prefix}/bin
%dir %{_mandir}
%dir %{_mandir}/man1
%{_prefix}/bin/%{spu_prefix}cc
%{_prefix}/bin/%{spu_prefix}c89
%{_prefix}/bin/%{spu_prefix}c99
%{_prefix}/bin/%{spu_prefix}gcc
%{_prefix}/bin/%{spu_prefix}gcov
%{_prefix}/bin/%{spu_prefix}cpp
%{_mandir}/man1/%{spu_prefix}gcc.1*
%{_mandir}/man1/%{spu_prefix}gcov.1*
%{_mandir}/man1/%{spu_prefix}cpp.1*
%dir %{_prefix}/libexec
%dir %{_prefix}/libexec/gcc
%dir %{_prefix}/libexec/gcc/%{spu_target_platform}
%dir %{_prefix}/libexec/gcc/%{spu_target_platform}/%{gcc_version}
%{_prefix}/libexec/gcc/%{spu_target_platform}/%{gcc_version}/cc1
%{_prefix}/libexec/gcc/%{spu_target_platform}/%{gcc_version}/collect2
%dir %{_prefix}/lib
%dir %{_prefix}/lib/gcc
%dir %{_prefix}/lib/gcc/%{spu_target_platform}
%dir %{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}
%dir %{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/stddef.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/stdarg.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/varargs.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/float.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/limits.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/stdbool.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/iso646.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/syslimits.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/unwind.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/spu_*.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/vmx2spu.h
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/README
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/crt*.o
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/libgcc.a
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/libgcov.a

%files -n spu-gcc-c++
%defattr(-,root,root)
%dir %{_prefix}
%dir %{_prefix}/bin
%dir %{_mandir}
%dir %{_mandir}/man1
%{_prefix}/bin/%{spu_prefix}g++
%{_prefix}/bin/%{spu_prefix}c++
%{_mandir}/man1/%{spu_prefix}g++.1*
%dir %{_prefix}/lib
%dir %{_prefix}/lib/gcc
%dir %{_prefix}/lib/gcc/%{spu_target_platform}
%dir %{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}
%dir %{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include
%dir %{_prefix}/libexec
%dir %{_prefix}/libexec/gcc
%dir %{_prefix}/libexec/gcc/%{spu_target_platform}
%dir %{_prefix}/libexec/gcc/%{spu_target_platform}/%{gcc_version}
%{_prefix}/libexec/gcc/%{spu_target_platform}/%{gcc_version}/cc1plus
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/libstdc++.a
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/libsupc++.a
%{_prefix}/lib/gcc/%{spu_target_platform}/%{gcc_version}/include/c++

%files -n spu-binutils
%defattr(-,root,root)
%dir %{_prefix}
%dir %{_prefix}/bin
%dir %{_mandir}
%dir %{_mandir}/man1
%{_prefix}/bin/embedspu.sh
%{_prefix}/bin/embedspu
%{_prefix}/bin/%{spu_prefix}addr2line
%{_prefix}/bin/%{spu_prefix}ar
%{_prefix}/bin/%{spu_prefix}as
%{_prefix}/bin/%{spu_prefix}c++filt
%{_prefix}/bin/%{spu_prefix}ld
%{_prefix}/bin/%{spu_prefix}nm
%{_prefix}/bin/%{spu_prefix}objcopy
%{_prefix}/bin/%{spu_prefix}objdump
%{_prefix}/bin/%{spu_prefix}ranlib
%{_prefix}/bin/%{spu_prefix}readelf
%{_prefix}/bin/%{spu_prefix}size
%{_prefix}/bin/%{spu_prefix}strings
%{_prefix}/bin/%{spu_prefix}strip
%dir %{_prefix}/%{spu_target_platform}
%dir %{_prefix}/%{spu_target_platform}/bin
%dir %{_prefix}/%{spu_target_platform}/lib
%{_prefix}/%{spu_target_platform}/bin/*
%{_prefix}/%{spu_target_platform}/lib/ldscripts
%{_mandir}/man1/%{spu_prefix}addr2line.1*
%{_mandir}/man1/%{spu_prefix}ar.1*
%{_mandir}/man1/%{spu_prefix}as.1*
%{_mandir}/man1/%{spu_prefix}c++filt.1*
%{_mandir}/man1/%{spu_prefix}ld.1*
%{_mandir}/man1/%{spu_prefix}nm.1*
%{_mandir}/man1/%{spu_prefix}objcopy.1*
%{_mandir}/man1/%{spu_prefix}objdump.1*
%{_mandir}/man1/%{spu_prefix}ranlib.1*
%{_mandir}/man1/%{spu_prefix}readelf.1*
%{_mandir}/man1/%{spu_prefix}size.1*
%{_mandir}/man1/%{spu_prefix}strings.1*
%{_mandir}/man1/%{spu_prefix}strip.1*

%files -n spu-newlib
%defattr(-,root,root)
%dir %{_prefix}
%dir %{_prefix}/%{spu_target_platform}
%dir %{_prefix}/%{spu_target_platform}/include
%dir %{_prefix}/%{spu_target_platform}/lib
%{_prefix}/%{spu_target_platform}/include/*.h
%{_prefix}/%{spu_target_platform}/include/machine/*.h
%{_prefix}/%{spu_target_platform}/include/sys/*.h
%{_prefix}/%{spu_target_platform}/lib/*.[ao]

%files -n spu-gdb
%defattr(-,root,root)
%dir %{_prefix}
%dir %{_prefix}/bin
%dir %{_mandir}
%dir %{_mandir}/man1
%{_prefix}/bin/%{spu_prefix}gdb*
%{_mandir}/man1/%{spu_prefix}gdb*

